@@ -46,9 +46,6 @@ class ionTrails(object):
         else:
             self.Ndens = 0
 
-        if verbose:
-            print('Temperature size:  %5i  Density size:  %5i'%(self.Ntemp, self.Ndens))
-
         self.NTempDens = max(self.Ndens,self.Ntemp)
         if self.Ndens > 1 and self.Ntemp == 1:
             self.Temperature = np.tile(self.Temperature, self.NTempDens)
@@ -77,8 +74,6 @@ class ionTrails(object):
                 raise ValueError('the size of em must be either 1 or the size of the larger of temperature or density %5i'%(self.NTempDens))
         else:
             self.Em = np.ones_like(self.Temperature, np.float64)
-        if verbose:
-            print('Em size:  %5i'%(self.Em.size))
 
     def intensityList(self, index=-1, wvlRange=None, wvlRanges=None, top=10, relative=0, outFile=0, rightDigits=4 ):
         """
@@ -214,10 +209,12 @@ class ionTrails(object):
             print('using index = %5i specifying temperature = %10.2e, eDensity =  %10.2e em = %10.2e'%(index, temperature[index], eDensity[index], em[index]))
             self.Message = 'using index = %5i specifying temperature = %10.2e, eDensity =  %10.2e = %10.2e'%(index, temperature[index], eDensity[index], em[index])
             intensity=intensity[index]
-
-        if wvlRange:
+        if wvlRange is None and wvlRanges is None:
+            wvlRange = self.WvlRange
             wvlIndex=util.between(wvl,wvlRange)
-        elif wvlRanges:
+        elif wvlRange is not None:
+            wvlIndex=util.between(wvl,wvlRange)
+        elif wvlRanges is not None:
             wvlIndex = []
             for awvlRange in wvlRanges:
                 wvlIndex.extend(util.between(wvl,awvlRange))
@@ -290,10 +287,10 @@ class ionTrails(object):
             outpt.close()
 
 
-    def intensityPlot(self, index=-1, wvlRange=None, top=10, linLog='lin', relative=False, verbose=False, plotFile=0, em=0 ):
+    def intensityPlot(self, index=-1, wvlRange=None, top=10, linLog='lin', relative=False, verbose=False, plotFile=0, em=0):
         """
         Plot the line intensities. Uses `Intensity` if it already exists. If
-        not, call the `intensity` method.
+        not, calls the `intensity` method.
 
         Parameters
         -----------
@@ -316,7 +313,7 @@ class ionTrails(object):
             if an Intensity attribute needs be created, then the emission measure is applied
         """
         if hasattr(self, 'Spectroscopic'):
-            title = self.Spectroscopic
+            title = 'Spectroscopic'
         else:
             title = ''
 
@@ -365,8 +362,8 @@ class ionTrails(object):
                 self.Message = 'using index = %5i specifying temperature = %10.2e, eDensity =  %10.2e'%(index, temperature[index], eDensity[index])
 
             intensity = self.Intensity['intensity'][index]
-            dstr=' -  Density = %10.2e (cm$^{-3}$)' % eDensity[index]
-            tstr=' -  T = %10.2e (K)' % temperature[index]
+            dstr=' Density = %10.2e (cm$^{-3}$)' % eDensity[index]
+            tstr=' T = %10.2e (K)' % temperature[index]
 
         if wvlRange is not None:
             wvlIndex = util.between(wvl, wvlRange)
@@ -392,7 +389,7 @@ class ionTrails(object):
 
         # must follow setting top
         plt.figure()
-        ylabel = 'intensity'
+        ylabel = 'Intensity'
         if relative:
             intensity = intensity/intensity[-1]
             ylabel += ' (Relative)'
