@@ -3,6 +3,7 @@ Base class used in several ChiantiPy objects
 """
 
 from datetime import datetime
+import pickle
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -28,7 +29,7 @@ class specTrails(object):
         #
         # ---------------------------------------------------------------------------
         #
-    def convolve(self, wavelength=0, filter=(chfilters.gaussianR, 1000.), label=0, verbose=0):
+    def convolve(self, wavelength=0, filter=(chfilters.gaussianR, 1000.), label=0, verbose=False):
         '''
         the first application of spectrum calculates the line intensities within the specified wavelength range and for set of ions specified
 
@@ -37,6 +38,23 @@ class specTrails(object):
         wavelength IS need for 'bunch' objects - in this case, the wavelength should not extend beyond the limits of the
         wvlRange used for the 'bunch' calculation
 
+        Keyword Arguments
+        -----------------
+
+        wavelength:  'int', `list`
+            if an `int`, the attribute 'Wavelength' is looked for
+            otherwise, wavelength is used
+
+        filter: `tuple`
+            first elements if one of the ChiantiPy.tools.filters object
+            second element is the width appropriate to the filter
+
+        label:  `str`
+            if set, creates a Spectrum[label] attribute
+
+        verbose: `bool`
+            if True, prints info to the terminal
+
         '''
         if not hasattr(self, 'IonInstances'):
             print(' must set keepIons=1 in order to keep self.IonInstances')
@@ -116,7 +134,7 @@ class specTrails(object):
         #
         # ---------------------------------------------------------------------------
         #
-    def ionGate(self, elementList = None, ionList = None, minAbund=None, doLines=1, doContinuum=1, doWvlTest=1, doIoneqTest=1,  verbose=0):
+    def ionGate(self, elementList = None, ionList = None, minAbund=None, doLines=1, doContinuum=1, doWvlTest=1, doIoneqTest=1, includeDiel=False,  verbose=0):
         '''
         creates a list of ions for free-free, free-bound, and line intensity calculations
         if doing the radiative losses, accept all wavelength -> doWvlTest=0
@@ -223,7 +241,7 @@ class specTrails(object):
         #
         #
         #  the relative H abundance is 1.0, the rest are all smaller
-        if minAbund < 2.:
+        if minAbund is not None and type(minAbund) is float:
             for iz in range(1, 31):
                 abundance = chdata.Abundance[self.AbundanceName]['abundance'][iz-1]
                 if abundance >= minAbund:
@@ -242,7 +260,10 @@ class specTrails(object):
                                 else:
                                     wvlTestMin = 1
                                     wvlTestMax = 1
-                            ioneqTest = (temperature.max() >= ionInfo[ionS]['tmin']) and (temperature.min() <= ionInfo[ionS]['tmax'])
+                            if temperature.size == 1:
+                                ioneqTest = (temperature >= ionInfo[ionS]['tmin']) and (temperature <= ionInfo[ionS]['tmax'])
+                            else:
+                                ioneqTest = (temperature.max() >= ionInfo[ionS]['tmin']) and (temperature.min() <= ionInfo[ionS]['tmax'])
                         # construct similar test for the dielectronic files
                         ionSd = util.zion2name(iz, ionstage, dielectronic=1)
                         masterListTestD = ionSd in masterlist
@@ -297,7 +318,18 @@ class specTrails(object):
         #  remove dupicates
 #        todoSet = set(todo)
 #        self.Todo = list(todoSet)
-        self.Todo = todo
+        dielList = []
+        if not includeDiel:
+            for akey in todo:
+                if 'd' in akey[-1]:
+                    dielList.append(akey)
+#                    if verbose:
+#                        print(' removed dielectronic ion %s for %s', akey,  stuff)
+        newTodo = {}
+        for akey in todo:
+            if akey not in dielList:
+                newTodo[akey] = todo[akey]
+        self.Todo = newTodo
         if len(self.Todo.keys()) == 0:
             print(' no elements have been selected')
             print(' it is necessary to provide an elementList, an ionList, or set minAbund > 0.')
@@ -305,18 +337,72 @@ class specTrails(object):
     #
     # -------------------------------------------------------------------------
     #
-    def spectrumPlot(self, index=-1, integrated=0, saveFile=0, linLog = 'lin'):
+    def saveData(self, filename):
+        """
+
+        :param filename: filename where the pickle file of the saved data will be stored
+        :type filename: str
+
+        following running a ch.spectrum calculation, save the calculation as a dictionary to a pickle file
+        """
+        data = {'temperature':self.Temperature, 'eDensity':self.EDensity,
+            'em':self.Em, 'abundanceName':self.AbundanceName, 'abundAll':self.AbundAll,
+            'ionsCalculated':self.IonsCalculated,
+            'defaults':self.Defaults, 'intensity':self.Intensity,
+            'nTemp':self.Ntemp, 'nDens':self.Ndens, 'nTempDens':self.NTempDens}
+
+#          'elementList':self.elementList, 'ionList':self.ionList,
+#            'minAbund':self.minAbund, 'keepIons':self.keepIons, 'em':self.em, 'abundance':self.abundance,
+#            'allLines':self.allLines}
+
+        if hasattr(self, 'Spectrum'):
+            data['spectrum'] = self.Spectrum
+        if hasattr(self, 'Wvl'):
+            data['wvl'] = self.Wvl
+        if hasattr(self, 'Wavelength'):
+            data['wavelength'] = self.Wavelength
+        if hasattr(self, 'WvlRange'):
+            data['wvlRange'] = self.WvlRange
+        if hasattr(self, 'PDensity'):
+            data['pDensity'] = self.PDensity
+        if hasattr(self, 'IonInstances'):
+            data['ionInstances'] = self.IonInstances
+        if hasattr(self, 'Xlabel'):
+            data['xlabel'] = self.Xlabel
+        if hasattr(self, 'Ylabel'):
+            data['ylabel'] = self.Ylabel
+        with open(filename, 'wb') as outpt:
+            pickle.dump(data, outpt)
+
+    def spectrumPlot(self, index=-1, integrated=False, saveFile=False, linLog = 'lin'):
         '''
         to plot the spectrum as a function of wavelength
+
+        Keyword Arguments
+        -----------------
+
+        index:  `int`
+            selects the temperature of the calculated spectrum
+
+        integrated:  `bool`
+            if True, plots the integrated/summed spectrum
+
+        saveFile:  `bool`, `str`
+            if set, saves the plot to 'saveFile'
+
+        linLog:  `str`
+            should be either 'lin' for linear, or 'log' for logarithmic base10
+
         '''
+        fs = 14
         plt.figure()
         mask = self.Em > 1.
         if mask.sum() == 0:
-            ylabel = r'erg cm$^{-2}$ s$^{-1}$ sr$^{-1} \AA^{-1}$ ($\int\,$ N$_e\,$N$_H\,$d${\it l}$)$^{-1}$'
+            self.Ylabel = r'erg cm$^{-2}$ s$^{-1}$ sr$^{-1} \AA^{-1}$ ($\int\,$ N$_e\,$N$_H\,$d${\it l}$)$^{-1}$'
         else:
-            ylabel = r'erg cm$^{-2}$ s$^{-1}$ sr$^{-1} \AA^{-1}$'
+            self.Ylabel = r'erg cm$^{-2}$ s$^{-1}$ sr$^{-1} \AA^{-1}$'
         #
-        xlabel = 'Wavelength ('+self.Defaults['wavelength'] +')'
+        self.Xlabel = 'Wavelength ('+self.Defaults['wavelength'] +')'
         #
 #        ymin = 10.**(np.log10(emiss.min()).round(0))
         #
@@ -327,9 +413,9 @@ class specTrails(object):
                 plt.plot(self.Spectrum['wavelength'], self.Spectrum['integrated'])
             elif 'wvl' in sorted(self.Spectrum.keys()):
                 plt.plot(self.Spectrum['wvl'], self.Spectrum['integrated'])
-            plt.xlabel(xlabel)
-            plt.ylabel(ylabel)
-            plt.title('integrated spectrum')
+            plt.xlabel(self.Xlabel,  fontsize=fs)
+            plt.ylabel(self.Ylabel,  fontsize=fs)
+            plt.title('integrated spectrum',  fontsize=fs)
         else:
             nTempDens = self.NTempDens
             if nTempDens == 1:
@@ -338,7 +424,7 @@ class specTrails(object):
                     plt.plot(self.Spectrum['wavelength'], self.Spectrum['intensity'])
                 elif 'wvl' in sorted(self.Spectrum.keys()):
                     plt.plot(self.Spectrum['wvl'], self.Spectrum['intensity'])
-                    plt.title(' Temperature = %10.2e K'%(self.Temperature))
+                    plt.title(' Temperature = %10.2e K'%(self.Temperature),  fontsize=fs)
             else:
                 if index < 0:
                     index = nTempDens/2
@@ -346,18 +432,36 @@ class specTrails(object):
                     plt.plot(self.Spectrum['wavelength'], self.Spectrum['intensity'][index])
                 elif 'wvl' in sorted(self.Spectrum.keys()):
                     plt.plot(self.Spectrum['wvl'], self.Spectrum['intensity'][index])
-                    plt.title(' Temperature = %10.2e K for index = %3i'%(self.Temperature[index], index))
-                plt.xlabel(xlabel)
-                plt.ylabel(ylabel)
+                    plt.title(' Temperature = %10.2e K for index = %3i'%(self.Temperature[index], index),  fontsize=fs)
+                plt.xlabel(self.Xlabel,  fontsize=fs)
+                plt.ylabel(self.Ylabel,  fontsize=fs)
+        ylim = plt.ylim()
+        plt.ylim([0., ylim[1]])
         plt.tight_layout()
         if saveFile:
             plt.savefig(saveFile)
     #
     # -------------------------------------------------------------------------
     #
-    def lineSpectrumPlot(self, index = 0, integrated=0, saveFile=0, linLog = 'lin'):
+    def lineSpectrumPlot(self, index = 0, integrated=False, saveFile=False, linLog = 'lin'):
         '''
         to plot the line spectrum as a function of wavelength
+
+        Keyword Arguments
+        -----------------
+
+        index:  `int`
+            selects the temperature of the calculated line spectrum
+
+        integrated:  `bool`
+            if True, plots the integrated/summed line spectrum
+
+        saveFile:  `bool`, `str`
+            if set, saves the plot to 'saveFile'
+
+        linLog:  `str`
+            should be either 'lin' for linear, or 'log' for logarithmic base10
+
         '''
         #
         #
