@@ -1,5 +1,7 @@
 """
 Reading and writing functions
+
+TODO:  see if klgfbRead is needed or not
 """
 import os
 from datetime import date
@@ -14,33 +16,48 @@ import ChiantiPy.Gui as chgui
 from  ChiantiPy.fortranformat import FortranRecordReader
 
 today = date.today()
-def abundanceRead(abundancename=''):
+
+def abundanceRead(abundancename='', verbose=False):
     """
     Read abundance file `abundancename` and return the abundance values relative to hydrogen
+
+    Keyword Arguments
+    -----------------
+
+    abundancename:  `str`
+        the name of the abundance file in the $XUVTOP/abundance directory to be read
+        the default is an empty string and then the 'default' abundance values are read
+
+    verbose:  `bool`
+        if true, prints out some info
+
     """
     abundance = np.zeros((50),np.float64)
     xuvtop = os.environ["XUVTOP"]
+    abundDir = os.path.join(xuvtop,'abundance')
+    abundList = os.listdir(abundDir)
     if abundancename:
         # a specific abundance file name has been specified
-        abundancefile = os.path.join(xuvtop,'abundance',abundancename+'.abund')
+        cnt = abundancename.count('.abund')
+        if cnt == 0:
+            abundancename += '.abund'
+    if abundancename in abundList:
+        abundanceFileName = os.path.join(abundDir,abundancename)
+        if verbose:
+            print('abundanceFileName: %s'%(abundanceFileName))
+
     else:
         # the user will select an abundance file
-        abundir = os.path.join(xuvtop,'abundance')
-        abundlabel = 'ChiantiPy - Select an abundance file'
-        #fname = chianti.gui.chpicker(abundir, filter='*.abund', label=abundlabel)
-        fname = chgui.gui.chpicker(abundir, filter='*.abund', label=abundlabel)
-        if fname is None:
+        abundLabel = 'ChiantiPy - Select an abundance file'
+        mypick = chgui.gui.selectorDialog(abundList, label=abundLabel)
+        abundName = mypick.selectedText[0]
+        if abundName is None:
             print((' no abundance file selected'))
             return 0
         else:
-            abundancefile = os.path.join(abundir, fname)
-            abundancefilename = os.path.basename(fname)
-            abundancename,ext = os.path.splitext(abundancefilename)
-#    else:
-#        # the default abundance file will be used
-#        abundancename=self.Defaults['abundfile']
-#        fname=os.path.join(xuvtop,'abundance',abundancename+'.abund')
-    input = open(abundancefile,'r')
+            abundanceFileName = os.path.join(abundDir,abundName)
+
+    input = open(abundanceFileName,'r')
     s1 = input.readlines()
     input.close()
     nlines = 0
@@ -58,7 +75,7 @@ def abundanceRead(abundancename=''):
     abs = 10.**(abundance[gz]-abundance[0])
     abundance.put(gz,abs)
     abundanceRef = s1[nlines+1:]
-    return {'abundancename':abundancename,'abundance':abundance,'abundanceRef':abundanceRef}
+    return {'abundancename':abundanceFileName,'abundance':abundance,'abundanceRef':abundanceRef}
 
 
 def zion2name(z,ion, dielectronic=False):
@@ -72,11 +89,22 @@ def zion2name(z,ion, dielectronic=False):
     ion : `int`
         the ion stage, for example, 14 for Fe XIV
 
+    Keyword Arguments
+    -----------------
+
+    dielectronic:  `bool`
+        if True, created the name of a dielectronic ion, with a 'd' at the end
+    Todo
+    ----
+
+    See if dielectronic is still appropriate
+
+    Put in separate module to avoid multiple copies
 
     Notes
     -----
     A duplicate of the routine in `ChiantiPy.tools.util` but needed by masterList Info
-    TODO: Put in separate module to avoid multiple copies
+
     """
     if ion == 0:
         thisone = 0
@@ -102,11 +130,15 @@ def convertName(name):
         a generic name of an ion in the CHIANTI database,
         such as fe_14 for Fe XIV
 
+    Todo
+    ----
+
+    Put in separate module to avoid multiple copies
 
     Notes
     -----
     A duplicate of the routine in `ChiantiPy.tools.util` but needed by masterList Info
-    TODO: Put in separate module to avoid multiple copies
+
     """
     s2 = name.split('_')
     els = s2[0].strip()
@@ -357,7 +389,30 @@ def defaultsRead(verbose=False):
     Read in configuration from .chiantirc file or set defaults if one is not found.
     """
     initDefaults = {'abundfile': 'sun_photospheric_2015_scott','ioneqfile': 'chianti', 'wavelength': 'angstrom', 'flux': 'energy','gui':False}
-    rcfile = os.path.join(os.environ['HOME'],'.chianti/chiantirc')
+
+    if "HOME" in os.environ.keys():
+        if os.path.isdir(os.environ["HOME"]):
+            home = os.environ["HOME"]
+            fn1 = os.path.join(home, '.chianti', 'chiantirc')
+            fn2 = os.path.join(home, '.config', 'chiantirc')
+            rcfile = False
+            if os.path.isfile(fn1):
+                rcfile = fn1
+            elif os.path.isfile(fn2):
+                rcfile = fn2
+    elif "PROFILEHOME" in os.environ.keys() and rcfile is False:
+        if os.path.isdir(os.environ["PROFILEHOME"]):
+            home = os.environ["PROFILEHOME"]
+            fn1 = os.path.join(home, '.chianti', 'chiantirc')
+            fn2 = os.path.join(home, '.config', 'chiantirc')
+            rcfile = False
+            if os.path.isfile(fn1):
+                rcfile = fn1
+            elif os.path.isfile(fn2):
+                rcfile = fn2
+    if verbose:
+        print('rcfile:  %s'%(rcfile))
+
     if os.path.isfile(rcfile):
         print((' reading chiantirc file'))
         config = configparser.RawConfigParser(initDefaults)
@@ -369,6 +424,7 @@ def defaultsRead(verbose=False):
             defaults['gui'] = True
         elif defaults['gui'].lower() in ('f', 'n', 'no', 'off', 'false', '0', 0, False):
             defaults['gui'] = False
+        defaults['rcfile'] = rcfile
     else:
         defaults = initDefaults
         if verbose:
@@ -385,8 +441,10 @@ def diRead(ions, filename=None):
 
     Parameters
     ----------
+
     ions : `str`
         Ion, e.g. 'c_5' for C V
+
     filename : `str`, optional
         Custom filename, will override that specified by `ions`
     """
@@ -439,12 +497,15 @@ def diRead(ions, filename=None):
         eaev = [float(avalue) for avalue in eacoef]
     else:
         eaev = 0.
+    ref = []
     hdr = input.readlines()
+    for ahdr in hdr:
+        ref.append(ahdr.strip())
     input.close()
     info = {"iz":iz,"ion":ion,"nspl":nspl,"neaev":neaev, 'nfac':nfac}
     if neaev:
         info['eaev'] = eaev
-    DiParams = {"info":info,"btf":btf,"ev1":ev1,"xsplom":xsplom,"ysplom":ysplom, 'eaev':eaev,"ref":hdr}
+    DiParams = {"info":info,"btf":btf,"ev1":ev1,"xsplom":xsplom,"ysplom":ysplom, 'eaev':eaev,"ref":ref}
     return DiParams
 
 
@@ -452,6 +513,15 @@ def drRead(ions, filename=None):
     """
     Read CHIANTI dielectronic recombination .drparams files
     if filename is set, then reads that file
+
+    Parameters
+    ----------
+
+    ions : `str`
+        Ion, e.g. 'c_5' for C V
+
+    filename : `str`, optional
+        Custom filename, will override that specified by `ions`
     """
     #
     #
@@ -548,6 +618,7 @@ def eaRead(ions, filename=None):
         header_line1 = FortranRecordReader('6x,3i3,8e10.0')
         header_line2 = FortranRecordReader('6x,3i3,12e10.0')
 
+
         for i in range(0,nsplups):
             try:
                 inpt = header_line1.read(s1[i])
@@ -561,18 +632,20 @@ def eaRead(ions, filename=None):
             gf[i] = inpt[3]
             de[i] = inpt[4]
             cups[i] = inpt[5]
-            if len(inpt)  > 13:
+            if len(inpt)  > 11:
                 nspl[i] = 9
                 splups[i].put(list(range(9)),inpt[6:])
             else:
                 nspl[i] = 5
                 splups[i].put(list(range(5)),inpt[6:])
         #
+        de = np.asarray(de,np.float64)
+        deEv = const.ryd2Ev*de
         ref = []
         for i in range(nsplups+1,len(s1)):
             s1a = s1[i][:-1]
             ref.append(s1a.strip())
-    return {"lvl1":lvl1,"lvl2":lvl2,"ttype":ttype,"gf":gf,"de":de,"cups":cups
+    return {"lvl1":lvl1,"lvl2":lvl2,"ttype":ttype,"gf":gf,"de":de, "deryd":de, "deEv":deEv, "cups":cups
                 ,"nspl":nspl,"splups":splups,"ref":ref}
 
 
@@ -763,11 +836,79 @@ def elvlcWrite(info, outfile=None, round=0, addLvl=0, includeRyd=False, includeE
     out.close()
     return
 
+def emRead(emName=''):
+    """
+    Read emission measure file `emName` and return the temperatures, densities and emission measures
+
+    Keyword Arguments
+    -----------------
+
+    emName:  `str`
+        the name of the emission measure file to read in the $XUVTOP/em directory
+
+    Returns
+    -------
+
+    EM:  `dict`
+        keywords are `temperature`, `density`, `em`, `filename`
+
+    """
+    xuvtop = os.environ["XUVTOP"]
+    emdir = os.path.join(xuvtop,'em')
+    emList = os.listdir(emdir)
+    temp = []
+    dens = []
+    em = []
+    if emName:
+        # a specific abundance file name has been specified
+        cnt = emName.count('.em')
+        if cnt == 0:
+            emName += '.em'
+    if emName in emList:
+        emFileName = os.path.join(xuvtop,'em',emName)
+    else:
+        # the user will select an emission measure file
+        emlabel = 'ChiantiPy - Select an emission measure file'
+        mypick = chgui.gui.selectorDialog(emList, label=emlabel)
+        emName = mypick.selectedText[0]
+        if emName is None:
+            print((' no EM file selected'))
+            return 0
+        else:
+            emName = mypick.selectedText[0]
+            emFileName = os.path.join(xuvtop,'em',emName)
+    input = open(emFileName,'r')
+    s1 = input.readlines()
+    input.close()
+    nlines = 0
+    idx = -1
+    while idx <= 0:
+        minChar = min([5, len(s1[nlines])])
+        aline = s1[nlines][0:minChar]
+        idx = aline.find('-1')
+        nlines += 1
+    nlines -= 1
+    for line in range(nlines):
+        a, b, c = s1[line].split()
+        temp.append(float(a))
+        dens.append(float(b))
+        em.append(float(c))
+    ref = s1[nlines+1:]
+    return{'temperature':np.asarray(temp), 'density':np.asarray(dens), 'em':np.asarray(em), 'ref':ref, 'filename':emFileName}
+
 
 def fblvlRead(ions, filename=None, verbose=False):
     """
     Read a Chianti energy level file for calculating the
     free-bound continuum
+
+    Parameters
+    ----------
+    ions : `str`
+        Ion, e.g. 'c_5' for C V
+    filename : `str`, optional
+        Custom filename, will override that specified by `ions`
+
     """
     fstring = 'i5,a20,2i5,a3,i5,2f20.3'
     header_line = FortranRecordReader(fstring)
@@ -946,43 +1087,43 @@ def itohRead():
     return {'itohCoef':gff}
 
 
-def klgfbRead():
-    """
-    Read CHIANTI files containing the free-bound gaunt factors for n=1-6 from [13]_.
-
-    Returns
-    -------
-    {'pe', 'klgfb'} : `dict`
-        Photon energy and the free-bound gaunt factors
-
-    References
-    ----------
-    .. [13] Karzas and Latter, 1961, ApJSS, `6, 167
-        <http://adsabs.harvard.edu/abs/1961ApJS....6..167K>`_
-    """
-    xuvtop = os.environ['XUVTOP']
-    fname = os.path.join(xuvtop, 'continuum', 'klgfb.dat')
-    input = open(fname)
-    lines = input.readlines()
-    input.close()
-    #
-    ngfb = int(lines[0].split()[0])
-    nume = int(lines[0].split()[1])
-
-    gfb = np.zeros((ngfb, ngfb, nume), np.float64)
-    nlines = len(lines)
-#        print 'nlines, nume, ngfb = ', nlines,  nume, ngfb
-    pe = np.asarray(lines[1].split(), np.float64)
-    for iline in range(2, nlines):
-        data = lines[iline].split()
-        n = int(data[0])
-        l = int(data[1])
-        gfb[n-1, l] = np.array(data[2:], np.float64)
-    return {'pe':pe, 'klgfb':gfb}
+#def klgfbRead():
+#    """
+#    Read CHIANTI files containing the free-bound gaunt factors for n=1-6 from [13]_.
+#
+#    Returns
+#    -------
+#    {'pe', 'klgfb'} : `dict`
+#        Photon energy and the free-bound gaunt factors
+#
+#    References
+#    ----------
+#    .. [13] Karzas and Latter, 1961, ApJSS, `6, 167
+#        <http://adsabs.harvard.edu/abs/1961ApJS....6..167K>`_
+#    """
+#    xuvtop = os.environ['XUVTOP']
+#    fname = os.path.join(xuvtop, 'continuum', 'klgfb.dat')
+#    input = open(fname)
+#    lines = input.readlines()
+#    input.close()
+#    #
+#    ngfb = int(lines[0].split()[0])
+#    nume = int(lines[0].split()[1])
+#
+#    gfb = np.zeros((ngfb, ngfb, nume), np.float64)
+#    nlines = len(lines)
+##        print 'nlines, nume, ngfb = ', nlines,  nume, ngfb
+#    pe = np.asarray(lines[1].split(), np.float64)
+#    for iline in range(2, nlines):
+#        data = lines[iline].split()
+#        n = int(data[0])
+#        l = int(data[1])
+#        gfb[n-1, l] = np.array(data[2:], np.float64)
+#    return {'pe':pe, 'klgfb':gfb}
 
 def klgbfnRead(filename):
     """
-    Read CHIANTI files containing the free-bound gaunt factors for n=1-6 from [13]_.
+    Read CHIANTI files containing the free-bound gaunt factors for n=1-6 from [14]_.
     This reads the corrected KL files in CHIANTI version 10+
 
     Parameters
@@ -998,7 +1139,7 @@ def klgbfnRead(filename):
 
     References
     ----------
-    .. [13] Karzas and Latter, 1961, ApJSS, `6, 167
+    .. [14] Karzas and Latter, 1961, ApJSS, `6, 167
         <http://adsabs.harvard.edu/abs/1961ApJS....6..167K>`_
     """
     kl = np.loadtxt(filename)
@@ -1028,7 +1169,18 @@ def maoParsRead(filename = None):
     62- 71  E10.4 ---     a2        Addiational fitting parameter
     73- 82  E10.4 ---     b2        Addiational fitting parameter
     84- 86  F3.1  ---     mdp       Maximum deviation in percent
+
+    Keyword Arguments
+    -----------------
+
+    filename:  `str`
+
+    Returns
+    -------
+
+    data: `dict`
     '''
+
     if filename is None:
         filename = os.path.join(os.environ['XUVTOP'], 'continuum', 'rrloss_mao_2017_pars.dat')
     else:
@@ -1039,9 +1191,7 @@ def maoParsRead(filename = None):
     with open(filename, 'r') as inpt:
         lines = inpt.readlines()
     nlines = len(lines)
-    print('len of pars %i'%(nlines))
     # this number is 255
-    print(lines[0])
     seq = [0]*nlines
     z = [0]*nlines
     a0 = [0]*nlines
@@ -1070,6 +1220,18 @@ def ioneqRead(ioneqName='', minIoneq=1.e-20, verbose=False):
     """
     Reads an ioneq file
     ionization equilibrium values less then minIoneq are returns as zeros
+
+    Keyword Arguments
+    -----------------
+
+    ioneqName:  `str`
+        reads the file in the $XUVTOP/ioneq directory, if a blank, the default is read
+
+    minIoneq:  `float`
+        sets values to zero if less the minIoneq
+
+    verbose:  `bool`
+        if true, prints into to the terminal
     Returns
     -------
     {'ioneqname','ioneqAll','ioneqTemperature','ioneqRef'} : `dict`
@@ -1196,6 +1358,15 @@ def masterListInfo(force=False, verbose=False):
     """
     Get information about ions in the CHIANTI masterlist.
 
+    Keyword Arguments
+    -----------------
+
+    force:  `bool`
+        if true, recreates the masterListInfo file
+
+    verbose:  `bool`
+        if true, prints into to the terminal
+
     Returns
     -------
     masterListInfo : `dict`
@@ -1230,8 +1401,8 @@ def masterListInfo(force=False, verbose=False):
         masterList = masterListRead()
         masterListInfo = {}
         haveZ = [0]*31
-        haveStage = np.zeros((31, 31), 'Int32')
-        haveDielectronic = np.zeros((31, 31), 'Int32')
+        haveStage = np.zeros((31, 31), np.int64)
+        haveDielectronic = np.zeros((31, 31), np.int64)
         for one in masterList:
             if verbose:
                 print((' ion = %s'%(one)))
@@ -1306,70 +1477,22 @@ def masterListInfo(force=False, verbose=False):
     return masterListInfo
 
 
-def photoxRead(ions):
-    """
-    Read CHIANTI photoionization .photox files
-
-    Returns
-    -------
-    {'lvl1', 'lvl2', 'energy', 'cross', 'ref'} : `dict`
-        Energy (in Rydbergs) and cross section (in :math:`\mathrm{cm}^{-2}`)
-
-    Notes
-    -----
-    The photox files are not in any released version of the CHIANTI database.
-    """
-    #
-    zion = util.convertName(ions)
-    if zion['Z'] < zion['Ion']:
-        print((' this is a bare nucleus that has no ionization rate'))
-        return
-    #
-    fname = util.ion2filename(ions)
-    paramname = fname+'.photox'
-    input = open(paramname,'r')
-    lines = input.readlines()
-    input.close
-    # get number of energies
-#    neng = int(lines[0][0:6])
-    dataEnd = 0
-    lvl1 = []
-    lvl2 = []
-    energy = []
-    cross = []
-    icounter = 0
-    while not dataEnd:
-        lvl11 = int(lines[icounter][:8])
-        lvl21 = int(lines[icounter][8:15])
-        ener = lines[icounter][15:].split()
-        energy1 = np.asarray(ener, np.float64)
-        #
-        icounter += 1
-        irsl = int(lines[icounter][:8])
-        ind0 = int(lines[icounter][8:15])
-        if irsl != lvl11 or ind0 != lvl21:
-            # this only happens if the file was written incorrectly
-            print((' lvl1, lvl2 = %7i %7i'%(lvl11, lvl21)))
-            print((' irsl, indo = %7i %7i'%(irsl,  ind0)))
-            return
-        crs = lines[icounter][15:].split()
-        cross1 = np.asarray(crs, np.float64)
-        lvl1.append(lvl11)
-        lvl2.append(lvl21)
-        energy.append(energy1)
-        cross.append(cross1)
-        icounter += 1
-        dataEnd = lines[icounter].count('-1')
-    ref = lines[icounter+1:-1]
-    cross = np.asarray(cross, np.float64)
-    energy = np.asarray(energy, np.float64)
-    return {'lvl1':lvl1, 'lvl2':lvl2,'energy':energy, 'cross':cross,  'ref':ref}
-
-
 def rrRead(ions, filename=None):
     """
     Read CHIANTI radiative recombination .rrparams files
 
+
+    Parameters
+    ----------
+    ions : `str`
+        Ion, e.g. 'c_5' for C V
+
+    Keyword Arguments
+    -----------------
+
+    filename : `str`, optional
+        Custom filename, will override that specified by `ions`
+
     Returns
     -------
     {'rrtype','params','ref'} : `dict`
@@ -1462,9 +1585,14 @@ def scupsRead(ions, filename=None, verbose=False):
     ----------
     ions : `str`
         Ion, e.g. 'c_5' for C V
+
+    Keyword Arguments
+    -----------------
+
     filename : `str`, optional
         Custom filename, will override that specified by `ions`
     verbose : `bool`
+        if True, prints info to the terminal
 
     '''
     #
@@ -1550,8 +1678,12 @@ def splomRead(ions, ea=False, filename=None):
     ----------
     ions : `str`
         Ion, e.g. 'c_5' for C V
+
+    Keyword Arguments
+    -----------------
+
     ea : `bool`
-        Read .easplom file
+        if true, reads the  .easplom file
     filename : `str`, optional
         Custom filename, will override that specified by `ions`
 
@@ -1572,19 +1704,25 @@ def splomRead(ions, ea=False, filename=None):
             splomname = fname+'.splom'
     else:
         splomname = filename
-    input = open(splomname,'r')
+    with open(splomname,'r') as inpt:
+        lines = inpt.readlines()
     #  need to read first line and see how many elements
-    line1 = input.readline()
     #indices=line1[0:15]
-    remainder = line1[16:]
+    remainder = lines[0][16:]
     nom = remainder.split(' ')
 #    format = FortranFormat('5i3,'+str(len(nom))+'E10.2')
     header_line = FortranRecordReader('5i3,'+str(len(nom))+'E10.2')
     #  go back to the beginning
-    input.seek(0)
-    lines = input.readlines()
-    data = 5
+    #  find the -1
+    marker = []
+    for iline,  aline in enumerate(lines):
+        if len(aline.split()) == 1:
+            marker.append(iline)
+        else:
+            pass
     iline = 0
+    z = []
+    stage = []
     lvl1 = []
     lvl2 = []
     ttype = []
@@ -1592,10 +1730,11 @@ def splomRead(ions, ea=False, filename=None):
     de = []
     f = []
     splom = []
-    #ntrans = 0
-    while data > 1:
+    for iline in range(marker[0]):
 #        splomdat = FortranLine(lines[iline],format)
         splomdat = header_line.read(lines[iline])
+        z.append(int(splomdat[0]))
+        stage.append(int(splomdat[1]))
         l1 = splomdat[2]
         l2 = splomdat[3]
         tt1 = splomdat[4]
@@ -1611,15 +1750,17 @@ def splomRead(ions, ea=False, filename=None):
         f.append(float(f1))
         splom.append(splom1)
         iline = iline+1
-        data = len(lines[iline].split(' ',2))
     hdr = lines[iline+1:-1]
+    ref = []
+    for ahdr in hdr:
+        ref.append(ahdr.strip())
     de = np.asarray(de,np.float64)
+    deEv = de*const.ryd2Ev
     splomout = np.asarray(splom,np.float64)
     splomout = np.transpose(splomout)
-    input.close()
     # note:  de is in Rydbergs
-    splom = {"lvl1":lvl1,"lvl2":lvl2,"ttype":ttype,"gf":gf,"deryd":de,"c":f
-        ,"splom":splomout,"ref":hdr}
+    splom = {"z":z, "stage":stage, "lvl1":lvl1,"lvl2":lvl2,"ttype":ttype,"gf":gf,"de":de, "deryd":de, "deEv":deEv,
+        "c":f,"splom":splomout,"ref":ref}
     return  splom
 
 
@@ -1631,6 +1772,9 @@ def splupsRead(ions, filename=None, filetype='splups'):
     ----------
     ions : `str`
         Ion, e.g. 'c_5' for C V
+
+    Keyword Arguments
+    -----------------
     filename : `str`, optional
         Custom filename, will override that specified by `ions`
     filetype : `str`, optional
@@ -1695,7 +1839,7 @@ def splupsRead(ions, filename=None, filetype='splups'):
             nspl[i] = len(as1)//10
 #            splupsFormat3 = FortranFormat(str(nspl[i])+'E10.2')
 #            splupsFormat3 = '(' + str(nspl[i]) + 'e10.3' + ')'
-            header_line3 = FortranRecordReader(str(nspl[i])+'e10.3' )
+            header_line3 = FortranRecordReader('%i'%(nspl[i])+'e10.3' )
 #            inpt = FortranLine(as1, splupsFormat3)
             inpt = header_line3.read(as1)
             spl1 = np.asarray(inpt[:], np.float64)
@@ -1717,28 +1861,6 @@ def splupsRead(ions, filename=None, filetype='splups'):
             return {"lvl1":lvl1,"lvl2":lvl2,"ttype":ttype,"gf":gf,"de":de,"cups":cups
                 ,"nspl":nspl,"splups":splups,"ref":ref, 'filename':splupsname}
 
-def trRead(ionS):
-    ''' read the files containing total recombination rates .trparams
-    '''
-    stuff = util.convertName(ionS)
-    filename = stuff['filename']
-    trname = filename + '.trparams'
-    if os.path.exists(trname):
-        temperature = []
-        rate = []
-        inpt = open(trname)
-        lines = inpt.readlines()
-        ndata = int(lines[0])
-        inpt.close()
-        for jline in range(1, ndata+1):
-            dummy = lines[jline].replace(os.linesep, '').split()
-            temperature.append(float(dummy[0]))
-            rate.append(float(dummy[1]))
-        return {'temperature':np.asarray(temperature, np.float64), 'rate':np.asarray(rate, np.float64)}
-    else:
-        return 'file does not exist'
-
-
 
 def twophotonHRead():
     """
@@ -1870,6 +1992,10 @@ def wgfaRead(ions, filename=None, elvlcname=0, total=False, verbose=False):
     ----------
     ions : `str`
         Ion, e.g. 'c_5' for C V
+
+    Keyword Arguments
+    -----------------
+
     filename : `str`
         Custom filename, will override that specified by `ions`
     elvlcname : `str`
